#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <sysexits.h>
#include <getopt.h>
#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

void usage(const char *pname) {
	fprintf(stderr, "Usage: %s [-h|--help] [-b|--boardid #] [-n|--name name] [-o|--output outfile]\n", pname);
	fprintf(stderr, "-h | --help:                 This usage statement\n");
	fprintf(stderr, "-b | --boardid:              A hexadecimal number for the board ID\n");
	fprintf(stderr, "-n | --name:                 The name of the board\n");
	fprintf(stderr, "-o | --output:               The filename to write the resource to\n");
	return;
}

int main(int argc, char *argv[]) {
	char c = 0;
	int loptind = 0;
	int32_t boardid = 0;
	char *name = NULL;
	char *outfile = NULL;
	int ofd;
	struct option o[] = {
		{"help", 0, 0, 'h'},
		{"name", 1, 0, 'n'},
		{"output", 1, 0, 'o'},
		{"boardid", 1, 0, 'b'},
		{0,0,0,0}
	};

	while( (c = getopt_long(argc, argv, "b:hn:o:", o, &loptind)) != -1 ) {
		switch(c) {
		case 'b':
			if(!optarg) {
				usage(argv[0]);
				exit(EX_USAGE);
			}
			sscanf(optarg, "%x", &boardid);
			break;
		case 'n':
			if(!optarg) {
				usage(argv[0]);
				exit(EX_USAGE);
			}
			name = optarg;
			break;
		case 'o':
			if(!optarg) {
				usage(argv[0]);
				exit(EX_USAGE);
			}
			outfile = optarg;
			break;
		case 'h':
			usage(argv[0]);
			exit(EX_USAGE);
			break;
		}
	}

	if(!name || !outfile || boardid == 0) {
		usage(argv[0]);
		exit(EX_USAGE);
	}
	errno = 0;

	ofd = open(outfile, O_WRONLY|O_CREAT|O_TRUNC, 0600);
	if(ofd < 0) {
		fprintf(stderr, "Could not open output file %s: %d %s\n", outfile, errno, strerror(errno));
		exit(EX_CONFIG);
	}
	
	uint32_t tmpint;
	uint32_t rsrctypeoff = 0;
	rsrctypeoff = 4 /* rsrc offset */
	            + 4 /* name offset */
	            + 4 /* board id */
	            + 4 /* EOL */;
	
	tmpint = 0x01000000 | rsrctypeoff;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Could not write sRsrcType offset: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}

	rsrctypeoff -= 4; /* decrement for next field */
	uint32_t rsrcnameoff = rsrctypeoff + 8 /* size of rsrctype */;
	tmpint = 0x02000000 | rsrcnameoff;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Could not write sRsrcName offset: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}

	tmpint = 0x20000000 | boardid;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Could not write BoardID: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}

	tmpint = 0xFF000000;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Could not write EOL: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}

	/* sRsrcType */
	tmpint = (0x0001 << 16) /* Category */ | 0x0000 /* cType */;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Could not write sRsrcType 1: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}
	tmpint = (0x0000 << 16) /* DrSW */ | 0x0000 /* DrHW */;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Could not write sRsrcType 2: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}
	
	/* sRsrcName */
	if(write(ofd, name, strlen(name)+1) < strlen(name)) {
		fprintf(stderr, "Could not write sRsrcName: %d %s\n", errno, strerror(errno));
		exit(EX_CONFIG);
	}

	close(ofd);
	exit(EX_OK);
}
