#ifndef __DECLROM_H__

#include <stdint.h>
#include <sys/queue.h>

#define kDeclRomPattern 0x5a932bc7

struct sRsrcType {
	uint16_t category;
	uint16_t cType;
	uint16_t drSW;
	uint16_t drHW;
};
typedef struct sRsrcType sRsrcType;

struct sBlock {
	uint32_t len;
	uint8_t *data;
};
typedef struct sBlock sBlock;

struct sExecBlock {
	uint32_t len;
	uint8_t revision;
	uint8_t cpuid;
	uint32_t offset;
	uint8_t *code;
};
typedef struct sExecBlock sExecBlock;

struct sDriver {
	uint32_t len;
	uint8_t *data;

	uint8_t cpuid;

	uint32_t nowrite;
};
typedef struct sDriver sDriver;

struct PixMap {
	uint32_t vpBaseOffset; /* Offset from base of frame buffer to start of page 0 */
	uint16_t vpRowBytes;   /* Width of each row of video memory (high bit clear) */
	uint16_t vpBounds[4];  /* BoundsRect for the video display (gives dimensions) */
	uint16_t vpVersion;    /* PixelMap version number (always 1) */
	uint16_t vpPackType;   /* Reserved */
	uint32_t vpPackSize;   /* Reserved */
	uint32_t vpHRes;       /* Horizontal resolution of the display device (pixels per inch) */
	uint32_t vpVRes;       /* Vertical resolution of the display device (pixels per inch) */
	uint16_t vpPixelType;  /* Defines pixel type ($0  chunky indexed; $10 = chunky direct) */
	uint16_t vpPixelSize;
	uint16_t vpCmpCount;
	uint16_t vpCmpSize;
	uint32_t vpPlaneBytes;
};
typedef struct PixMap PixMap;

struct sRsrcCLUT {
	uint32_t reclen;
	uint32_t ctSeed;
	uint16_t ctFlags;
	uint16_t ctSize;
	uint16_t white[4];
	uint16_t black[4];
};
typedef struct sRsrcCLUT sRsrcCLUT;

struct sRsrcVidMode {
	PixMap    *vidParams;
	sRsrcCLUT clut;
	uint32_t  pages;
	uint32_t  devtype;

	uint8_t modeid;

	TAILQ_ENTRY(sRsrcVidMode) next;
};
typedef struct sRsrcVidMode sRsrcVidMode;

struct sRsrcVidNameEntry {
	uint32_t reclen;
	uint16_t resid;
	char *name;

	uint8_t modeid;

	TAILQ_ENTRY(sRsrcVidNameEntry) next;
};
typedef struct sRsrcVidNameEntry sRsrcVidNameEntry;

struct sResource {
	uint8_t rsrcid;
	/* required */
	sRsrcType *type;
	char *name;
	
	/* optional */
	sDriver *drvrDir[4];
	sExecBlock *loadRec;
	sExecBlock *bootRec;
	uint32_t nowriteBootRec;
	uint32_t flags;
	uint32_t devid;;
	
	uint16_t boardid;
	sExecBlock *primaryInit;
	
	char *vendorid;
	char *serialnum;
	char *revlevel;
	char *partnum;
	char *date;

	uint32_t *minBaseOS;
	uint32_t *minLen;
	sExecBlock *secondaryInit;

	uint32_t *unknowns;
	uint32_t unknowncount;

	sRsrcVidMode vidMode;

	int8_t hasVidNames;
	TAILQ_HEAD(sRsrcVidNameDir, sRsrcVidNameEntry) vidNames;
	TAILQ_HEAD(sRsrcVidModeDir, sRsrcVidMode) vidModes;

	TAILQ_ENTRY(sResource) next;
};
typedef struct sResource sResource;

struct sFormatBlock {
	uint8_t bytelanes;
	uint8_t reserved;
	uint32_t testpattern;
	uint8_t format;
	uint8_t revision;
	uint32_t crc;
	uint32_t length;
	int32_t diroffset;
};
typedef struct sFormatBlock sFormatBlock;

struct sDeclRom {
	sFormatBlock	formatblock;
	TAILQ_HEAD(rsrcq, sResource) resourcelist;
	uint8_t *rom;
	uint32_t romlen;
};
typedef struct sDeclRom sDeclRom;

/* Enums for category types */
enum {
	catBoard = 1,
	catTest,
	catDisplay,
	catNetwork
};

void sdeclrom_free(sDeclRom *rom);
int sdeclrom_writefile(sDeclRom *rom, char *file);
sDeclRom *sdeclrom_readfile(char *infile);

#endif /* __DECLROM_H__ */
