#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <sysexits.h>
#include <getopt.h>
#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

void usage(const char *pname) {
	fprintf(stderr, "Usage: %s [-h|--help] [-d|--drvr driverfile] [-o|--output outfile]\n", pname);
	fprintf(stderr, "-h | --help:                 This usage statement\n");
	fprintf(stderr, "-c | --code:                 The filename for the code file\n");
	fprintf(stderr, "-o | --output:               The filename to write the resource to\n");
	return;
}

int main(int argc, char *argv[]) {
	char c = 0;
	int loptind = 0;
	char *infile = NULL;
	char *outfile = NULL;
	int ifd;
	int ofd;
	struct option o[] = {
		{"help", 0, 0, 'h'},
		{"code", 1, 0, 'c'},
		{"output", 1, 0, 'o'},
		{0,0,0,0}
	};

	while( (c = getopt_long(argc, argv, "c:ho:", o, &loptind)) != -1 ) {
		switch(c) {
		case 'c':
			if(!optarg) {
				usage(argv[0]);
				exit(EX_USAGE);
			}
			infile = optarg;
			break;
		case 'o':
			if(!optarg) {
				usage(argv[0]);
				exit(EX_USAGE);
			}
			outfile = optarg;
			break;
		case 'h':
			usage(argv[0]);
			exit(EX_USAGE);
			break;
		}
	}

	if(!infile || !outfile) {
		usage(argv[0]);
		exit(EX_USAGE);
	}
	errno = 0;

	ifd = open(infile, O_RDONLY);
	if(ifd < 0) {
		fprintf(stderr, "Could not open driver input file %s: %d %s\n", infile, errno, strerror(errno));
		exit(EX_CONFIG);
	}
	struct stat sb;
	fstat(ifd, &sb);

	ofd = open(outfile, O_WRONLY|O_CREAT|O_TRUNC, 0600);
	if(ofd < 0) {
		fprintf(stderr, "Could not open output file %s: %d %s\n", outfile, errno, strerror(errno));
		exit(EX_CONFIG);
	}
	
	uint32_t tmpint;
	tmpint = sb.st_size + 4 + 1 + 1 + 2 + 4;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Error writing SExecBlock block size: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}

	uint8_t tmpbyte;
	tmpbyte = 2;
	if(write(ofd, &tmpbyte, sizeof(tmpbyte)) != sizeof(tmpbyte)) {
		fprintf(stderr, "Error writing SExecBlock revision: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}
	tmpbyte = 2;
	if(write(ofd, &tmpbyte, sizeof(tmpbyte)) != sizeof(tmpbyte)) {
		fprintf(stderr, "Error writing SExecBlock CPU ID: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}

	uint16_t tmpword = 0;
	if(write(ofd, &tmpword, sizeof(tmpword)) != sizeof(tmpword)) {
		fprintf(stderr, "Error writing SExecBlock reserved space: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}

	tmpint = 4;
	tmpint = htonl(tmpint);
	if(write(ofd, &tmpint, sizeof(tmpint)) != sizeof(tmpint)) {
		fprintf(stderr, "Error writing SExecBlock offset: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}

	char *buffer = calloc(1, sb.st_size);

	if(read(ifd, buffer, sb.st_size) != sb.st_size) {
		fprintf(stderr, "Error reading code file: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}

	if(write(ofd, buffer, sb.st_size) != sb.st_size) {
		fprintf(stderr, "Error writing sExecBlock code: %d %s\n", errno, strerror(errno));
		exit(EX_SOFTWARE);
	}

	free(buffer);
	close(ifd);
	close(ofd);
	exit(EX_OK);
}
